﻿using FlowerShopBusinessLogic.BusinessLogic;
using FlowerShopBusinessLogic.MailWorker;

using FlowerShopContracts.BusinessLogicsContracts;
using FlowerShopContracts.StoragesContracts;
using FlowerShopDatabaseImplement.Implements;
using ProjectFlowerShop;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using System;
using System.Drawing;
using FlowerShopBusinessLogic;
using FlowerShopBusinessLogic.OfficePackage.Implements;
using FlowerShopBusinessLogic.OfficePackage;
using FlowerShopContracts.BindingModels;

namespace ProjectFlowerShop
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();
			try
			{
				var mailSender =
				_serviceProvider.GetService<AbstractMailWorker>();
				mailSender?.MailConfig(new MailConfigBindingModel
				{
					MailLogin =
				System.Configuration.ConfigurationManager.AppSettings["MailLogin"] ??
				string.Empty,
					MailPassword =
				System.Configuration.ConfigurationManager.AppSettings["MailPassword"] ??
				string.Empty,
					SmtpClientHost =
				System.Configuration.ConfigurationManager.AppSettings["SmtpClientHost"] ??
				string.Empty,
					SmtpClientPort =
				Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["SmtpClientPort"]),
					PopHost =
				System.Configuration.ConfigurationManager.AppSettings["PopHost"] ?? string.Empty,
					PopPort =
				Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["PopPort"])
				});

				var timer = new System.Threading.Timer(new
				TimerCallback(MailCheck!), null, 0, 100000);
			}
			catch (Exception ex)
			{
				var logger = _serviceProvider.GetService<ILogger>();
				logger?.LogError(ex, "Ошибка работы с почтой");
			}
			Application.Run(_serviceProvider.GetRequiredService<MainForm>());
        }
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IComponentStorage, ComponentStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IFlowerStorage, FlowerStorage>();
            services.AddTransient<IClientStorage, ClientStorage>();
			services.AddTransient<IImplementerStorage, ImplementerStorage>();
			services.AddTransient<IMessageInfoStorage, MessageInfoStorage>();

			services.AddTransient<IComponentLogic, ComponentLogic>();
			services.AddTransient<IMessageInfoLogic, MessageInfoLogic>();
			services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<IFlowerLogic, FlowerLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();
			services.AddTransient<IImplementerLogic, ImplementerLogic>();
			services.AddTransient<IClientLogic, ClientLogic>();
			services.AddTransient<IWorkProcess, WorkProcess>();

			services.AddTransient<MainForm>();
			services.AddTransient<ComponentForm>();
            services.AddTransient<FormComponents>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormFlower>();
            services.AddTransient<FormFlowerComponent>();
            services.AddTransient<FormFlowers>();
            services.AddTransient<FormClients>();
			services.AddTransient<ImplementersForm>();
			services.AddTransient<ImplementerForm>();
			services.AddTransient<FormReportFlowerComponent>();
            services.AddTransient<FormReportOrders>();
			services.AddTransient<FormViewMail>();

			services.AddSingleton<AbstractMailWorker, MailKitWorker>();
			services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();
        }
		private static void MailCheck(object obj) =>ServiceProvider?.GetService<AbstractMailWorker>()?.MailCheck();

	}
}
